@@ -133,7 +133,11 @@ static int create_nameip(nameip_t *nameip, char *ip)
     }
 
     nameip->ipnum = ntohl(ipnum.s_addr);
-    ipnum.s_addr = nameip->ipnum;
+    // Swap the bytes.  Don't assume 32-bit integers.
+    ipnum.s_addr = ((nameip->ipnum & 0xff000000 >> 24) |
+		    (nameip->ipnum & 0x00ff0000 >> 8) |
+		    (nameip->ipnum & 0x0000ff00 << 8) |
+		    (nameip->ipnum & 0x000000ff << 24));
 
     mkstring(&nameip->arpa, inet_ntoa(ipnum));
     
@@ -528,7 +532,8 @@ static int compile_namepointer(dnsheader_t *x, int offset)
 
 static unsigned char *compile_int(dnsheader_t *x, int number)
 {
-    *((unsigned short int *) x->here) = htons((unsigned short) number);
+    unsigned short conv = htons((unsigned short) number);
+    memcpy(x->here, &conv, 2);
     x->here += 2;
 
     return (x->here);
@@ -536,7 +541,9 @@ static unsigned char *compile_int(dnsheader_t *x, int number)
 
 static unsigned char *compile_long(dnsheader_t *x, long number)
 {
-    *((unsigned long *) x->here) = htonl((unsigned long) number);
+    unsigned long conv = htonl((unsigned long) number);
+    memcpy(x->here, &conv, 4);
+
     x->here += 4;
 
     return (x->here);
@@ -552,11 +559,14 @@ static unsigned char *compile_objectname(dnsheader_t *x)
 
 static int end_rdata(dnsheader_t *x)
 {
+    unsigned short int conv;
+
     if (x->rdata != NULL) {
 	int	rsize;
 
 	rsize = x->here - (x->rdata + 2);
-	*((unsigned short int *) x->rdata) = htons((unsigned short int) rsize);
+	conv = htons((unsigned short int) rsize);
+	memcpy(x->rdata, &conv, 2);
     }
     
     x->rdata = NULL;
@@ -579,7 +589,7 @@ static int end_assembly(dnsheader_t *x)
 /*  x->u.b.authorative = 0; */
 
     pkt = (unsigned short int *) x->packet;
-    pkt[1] = htons(x->u.flags);
+    pkt[1] = x->u.flags;
     pkt[2] = htons(x->qdcount);
     pkt[3] = htons(x->ancount);
     pkt[4] = htons(x->nscount);
